@@ -15,6 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 #include <gnome.h>
 #include "gnome-ppp.h"
 #include "account-window.h"
@@ -22,13 +23,22 @@
 #include "global.h"
 #include "misc.h"
 
+/* for zvtterm */
+#include <zvt/zvtterm.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkx.h>
+#include <gdk/gdkprivate.h>
+#include <gdk/gdkkeysyms.h>
+
+#define FONT "-misc-fixed-medium-r-normal--12-200-75-75-c-100-iso8859-1"
+#define SCROLLBACK_LINES 50
+
 
 enum
 {
   _DISCONNECTED,
   _IN_PROGRESS,
-  _CONNECTED,
-  _ERROR
+  _CONNECTED
 };
 
 
@@ -40,6 +50,12 @@ typedef struct _Dial
   GtkWidget *appbar;
   GtkWidget *connect_button_label;
   GtkWidget *connect;
+  GtkWidget *debug_terminal_check_menu_item;
+
+  GtkWidget *term_app;
+  GtkWidget *term;
+
+  GtkWidget *input_dialog;
 
   Account *account;
   gint state;
@@ -47,7 +63,6 @@ typedef struct _Dial
 } Dial;
 
 
-
 static void refresh_dial_windows();
 static void refresh_dial_app(Dial *dial);
 static void refresh_dial_account_list(Dial *dial);
@@ -56,19 +71,21 @@ static gint close_dial_app(GtkWidget *widget, Dial *dial);
 
 /* app callbacks */
 static void destroy_dial_app_cb(GtkWidget *widget);
-static void account_list_select_cb(GtkWidget *widget, gint row, gint col, 
-				   GdkEventButton *bevent);
 static void connect_button_cb(GtkWidget *widget);
 
 
 /* dial engine callbacks */
 static gint timeout_cb(Dial *dial);
-static void connect_message_cb(Account *account, PPPMessageType message, gchar *text, gpointer data);
+static void connect_message_cb(Account         *account, 
+			       PPPMessageType   message,
+			       gchar           *text, 
+			       gpointer         data);
 
 
 /* menu callbacks */
 static void menu_file_new_dialer_cb(GtkWidget *widget, Dial *dial);
 static void menu_file_exit_cb(GtkWidget *widget, Dial *dial);
+static void menu_view_debug_terminal_cb(GtkWidget *widget, Dial *dial);
 static void menu_account_new_cb(GtkWidget *widget, Dial *dial);
 static void menu_account_duplicate_cb(GtkWidget *widget, Dial *dial);
 static void menu_account_edit_cb(GtkWidget *widget, Dial *dial);
@@ -81,24 +98,49 @@ static Dial *malloc_dial();
 static void free_dial(Dial *dial);
 static Dial *get_dial_context(GtkObject *object);
 static void set_dial_context(GtkObject *object, Dial *dial);
+static Account *get_selected_account(Dial *dial);
+
+
+/* terminal stuff */
+static void new_terminal_app(Dial *dial);
+static void terminal_show(Dial *dial);
+static void terminal_hide(Dial *dial);
+static void terminal_write(Dial *dial, gchar *buff);
+
+static void terminal_size_allocate_cb(GtkWidget *widget);
+static gint terminal_delete_event_cb(GtkWidget *widget);
+static void terminal_menu_close_cb(GtkWidget *widget, Dial *dial);
+
+
+/* input dialog */
+static void input_dialog_destroy_cb(GtkWidget *widget);
+static void input_dialog_cb(gchar *string, gpointer data);
 
 
 /* menus */
 static GnomeUIInfo file_menu[] = 
 {
-  GNOMEUIINFO_MENU_NEW_ITEM("New Dialer", NULL, menu_file_new_dialer_cb, NULL),
+  GNOMEUIINFO_MENU_NEW_ITEM(N_("New Dialer"), NULL, menu_file_new_dialer_cb, NULL),
   GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_MENU_CLOSE_ITEM(menu_file_exit_cb, NULL),
   GNOMEUIINFO_MENU_EXIT_ITEM(menu_file_exit_cb, NULL),
   GNOMEUIINFO_END
 };
 
 
+static GnomeUIInfo view_menu[] =
+{
+  GNOMEUIINFO_TOGGLEITEM(N_("_Debug Terminal"), NULL, menu_view_debug_terminal_cb, NULL),
+  GNOMEUIINFO_END
+};
+
+
 static GnomeUIInfo account_menu[] = 
 {
-  GNOMEUIINFO_ITEM_NONE("New...", NULL, menu_account_new_cb),
-  GNOMEUIINFO_ITEM_NONE("Duplicate...", NULL, menu_account_duplicate_cb),
-  GNOMEUIINFO_ITEM_NONE("Edit...", NULL, menu_account_edit_cb),
-  GNOMEUIINFO_ITEM_NONE("Delete...", NULL, menu_account_delete_cb),
+  GNOMEUIINFO_ITEM_NONE(N_("New..."), NULL, menu_account_new_cb),
+  GNOMEUIINFO_ITEM_NONE(N_("Duplicate..."), NULL, menu_account_duplicate_cb),
+  GNOMEUIINFO_ITEM_NONE(N_("Edit..."), NULL, menu_account_edit_cb),
+  GNOMEUIINFO_ITEM_NONE(N_("Delete..."), NULL, menu_account_delete_cb),
   GNOMEUIINFO_END
 };
 
@@ -114,6 +156,7 @@ static GnomeUIInfo help_menu[] =
 static GnomeUIInfo main_menu[] =
 {
   GNOMEUIINFO_MENU_FILE_TREE(file_menu),
+  GNOMEUIINFO_MENU_VIEW_TREE(view_menu),
 
   { GNOME_APP_UI_SUBTREE, N_("_Account"), NULL, account_menu,
     NULL, NULL, (GnomeUIPixmapType) 0, NULL, 0, (GdkModifierType) 0,
@@ -148,11 +191,12 @@ new_dial_app()
   dial = malloc_dial();
 
   /* set up the main application */
-  dial->app = gnome_app_new("gnome-ppp", "gnome-ppp");
+  dial->app = gnome_app_new(GNOME_PPP_NAME, _("gnome-ppp"));
   set_dial_context(GTK_OBJECT(dial->app), dial);
 
   /* menus & status bar are imbedded into the gnome-app */
   gnome_app_create_menus_with_data(GNOME_APP(dial->app), main_menu, dial);
+  dial->debug_terminal_check_menu_item = view_menu[0].widget;
 
   dial->appbar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
   gnome_app_set_statusbar(GNOME_APP(dial->app), GTK_WIDGET(dial->appbar));
@@ -160,13 +204,13 @@ new_dial_app()
   gtk_signal_connect(
       GTK_OBJECT(dial->app),
       "destroy",
-     (GtkSignalFunc) destroy_dial_app_cb,
+      GTK_SIGNAL_FUNC(destroy_dial_app_cb),
       NULL);
 
   gtk_signal_connect(
       GTK_OBJECT(dial->app),
       "delete_event",
-     (GtkSignalFunc) close_dial_app,
+      GTK_SIGNAL_FUNC(close_dial_app),
       NULL);
 
   /* hbox */
@@ -188,13 +232,6 @@ new_dial_app()
 
   gtk_clist_column_titles_passive(GTK_CLIST(dial->account_clist));
   gtk_clist_set_selection_mode(GTK_CLIST(dial->account_clist), GTK_SELECTION_BROWSE);
-
-  gtk_signal_connect(
-      GTK_OBJECT(dial->account_clist),
-      "select_row",
-      (GtkSignalFunc) account_list_select_cb,
-      NULL);
-
   gtk_widget_show(dial->account_clist);
 
   /* connect button */
@@ -207,16 +244,19 @@ new_dial_app()
   gtk_box_pack_start(GTK_BOX(vbox), dial->connect, FALSE, FALSE, 0);
 
   gtk_container_add(GTK_CONTAINER(dial->connect), dial->connect_button_label);
-  gtk_widget_set_usize(dial->connect, 100, GPPP_BUTTON_HEIGHT);
+  gtk_widget_set_usize(dial->connect, 100, BUTTON_HEIGHT);
 
   gtk_signal_connect(
       GTK_OBJECT(dial->connect),
       "clicked",
-     (GtkSignalFunc) connect_button_cb,
+      GTK_SIGNAL_FUNC(connect_button_cb),
       NULL);
 
   gtk_widget_show(dial->connect);
 
+  /* create terminal */
+  new_terminal_app(dial);
+
   /* set data in the dial application */
   refresh_dial_account_list(dial);
   refresh_dial_app(dial);
@@ -247,17 +287,10 @@ static void
 refresh_dial_account_list(Dial *dial)
 {
   GList *list;
-  Account *account;
+  Account *saved_account, *account;
 
-  /* handle setting the default account context correctly */
-  if (g_list_length(account_list()) == 0)
-    {
-      dial->account = NULL;
-    }
-  else if (!dial->account || !g_list_find(account_list(), dial->account))
-    {
-      dial->account = (Account *) account_list()->data;
-    }
+  /* save selected account */
+  saved_account = get_selected_account(dial);
 
   /* refresh the accounts clist */
   gtk_clist_freeze(GTK_CLIST(dial->account_clist));
@@ -276,12 +309,12 @@ refresh_dial_account_list(Dial *dial)
       row = gtk_clist_append(GTK_CLIST(dial->account_clist), text);
       if (row < 0)
 	{
-	  g_error("refresh_dial_app(): gtk_clist_append failed.");
+	  g_error(_("refresh_dial_app(): gtk_clist_append failed."));
 	}
       gtk_clist_set_row_data(GTK_CLIST(dial->account_clist), row, account);
 
       /* select this row if it is the default account */
-      if (account == dial->account)
+      if (account == saved_account)
 	{
 	  gtk_clist_select_row(GTK_CLIST(dial->account_clist), row, 0);
 	}
@@ -296,38 +329,29 @@ refresh_dial_app(Dial *dial)
   /* set the active state of the connection button and the
    * account-selection option menu based on the connection state
    */
-  if (dial->account)
+  if (account_list())
     {
       switch (dial->state)
 	{
 	case _DISCONNECTED:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), "Connect");
+	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Connect"));
 	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->account_clist), TRUE);
-	  break;
-	  
-	case _ERROR:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), "---");
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), FALSE);
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->account_clist), FALSE);
 	  break;
 
 	case _IN_PROGRESS:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), "Cancel");
+	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Cancel"));
 	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->account_clist), FALSE);
 	  break;
 
 	case _CONNECTED:
-	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), "Disconnect");
+	  gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Disconnect"));
 	  gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), TRUE);
-	  gtk_widget_set_sensitive(GTK_WIDGET(dial->account_clist), FALSE);
 	  break;
 	}
     }
   else
     {
-      gtk_label_set_text(GTK_LABEL(dial->connect_button_label), "Connect");
+      gtk_label_set_text(GTK_LABEL(dial->connect_button_label), _("Connect"));
       gtk_widget_set_sensitive(GTK_WIDGET(dial->connect), FALSE);
     }
 }
@@ -356,6 +380,7 @@ destroy_dial_app_cb(GtkWidget *widget)
 
   dial = get_dial_context(GTK_OBJECT(widget));
 
+  gtk_widget_destroy(dial->term_app);
   free_dial(dial);
 
   /* quit gnome-ppp when there are no moer dial applications */
@@ -367,30 +392,6 @@ destroy_dial_app_cb(GtkWidget *widget)
 }
 
 
-/* callback for clist */
-static void 
-account_list_select_cb(GtkWidget *widget, gint row, gint col, GdkEventButton *bevent)
-{
-  Account *account;
-  Dial *dial;
-
-  dial = get_dial_context(GTK_OBJECT(widget));
-
-  account = gtk_clist_get_row_data(GTK_CLIST(widget), row);
-  if (account == NULL)
-    {
-      return;
-    }
-
-  dial->account = account;
-
-  if (!g_list_find(account_list(), account))
-    {
-      g_error("account_list_select_cb(): account from clist_date not in account list");
-    }
-}
-
-
 static void
 connect_button_cb(GtkWidget *widget)
 {
@@ -398,45 +399,42 @@ connect_button_cb(GtkWidget *widget)
 
   dial = get_dial_context(GTK_OBJECT(widget));
 
-  /* we need an account selected to connect */
-  if (!g_list_find(account_list(), dial->account))
-    {
-      g_error("internal error !g_list_find(account_list(), dial->account");
-    }
-
   switch (dial->state)
     {
     case _DISCONNECTED:
-      /* locked accounts are already in use */
-      if (dial->account->locked)
+      g_assert(dial->account == NULL);
+      g_assert(GTK_CLIST(dial->account_clist)->selection);
+
+      /* get selected account */
+      if (!(dial->account = get_selected_account(dial)))
 	{
-	  gnome_error_dialog(_("This account is already in use."));
-	}
-      else
-	{
-	  dial->account->locked = TRUE;
+	  g_error(_("connect_button_cb(): no selected account"));
 	}
 
-      /* start the connection; if the start is successful, then
-       * add the timeout callback to run the state engine
-       */
-      if (connect_start(dial->account, connect_message_cb, dial))
+      /* locked accounts are already in use */
+      if (!account_is_locked(dial->account))
 	{
-	  dial->timer_id = 
-	    gtk_timeout_add(500, (GtkFunction) timeout_cb, dial);
+	  account_lock(dial->account);
+
+	  /* start the connection; if the start is successful, then
+	   * add the timeout callback to run the state engine
+	   */
+	  if (connect_start(dial->account, connect_message_cb, dial))
+	    {
+	      dial->timer_id = 
+		gtk_timeout_add(500, (GtkFunction) timeout_cb, dial);
+	    }
 	}
       else
 	{
-	  dial->account->locked = FALSE;
+	  gnome_error_dialog(_("This account is in use."));
+	  dial->account = NULL; 
 	}
     break;
 
-    case _ERROR:
-      g_error("connect_button_cb(): called while in ERROR state");
-      break;
-
     case _IN_PROGRESS:
     case _CONNECTED:
+      g_assert(dial->account);
       connect_stop(dial->account);
     break;
     }
@@ -463,24 +461,25 @@ timeout_cb(Dial *dial)
 
 /* message callback from connect */
 static void
-connect_message_cb(Account *account, PPPMessageType message, gchar *text, gpointer data)
+connect_message_cb(Account         *account, 
+		   PPPMessageType   message, 
+		   gchar           *text, 
+		   gpointer         data)
 {
+  gboolean update_statusbar;
   gint saved_state;
   Dial *dial;
 
   dial = (Dial *) data;
+  update_statusbar = FALSE;
+  saved_state = dial->state;
 
   /* sanity checks */
-  g_assert(account != NULL);
   if (dial->account != account)
     {
-      g_error("connect_message_cb(): account=%s but dial->account=%s",
-          account->name->str,
-	  dial->account->name->str);
+      g_error("connect_message_cb(): account != dial->account");
     }
 
-  saved_state = dial->state;
-
   /* map messages from the connect engine to internal
    * states of the dial window
    */
@@ -488,42 +487,89 @@ connect_message_cb(Account *account, PPPMessageType message, gchar *text, gpoint
     {
     case PPP_DISCONNECTED:
       dial->state = _DISCONNECTED;
-      break;
-
-    case PPP_ERROR:
-      dial->state = _ERROR;
-      gnome_error_dialog(text);
+      update_statusbar = TRUE;
       break;
 
     case PPP_IN_PROGRESS:
       dial->state = _IN_PROGRESS;
+      update_statusbar = TRUE;
       break;
 
     case PPP_CONNECTED:
       dial->state = _CONNECTED;
+      update_statusbar = TRUE;
+      break;
+
+    case PPP_ERROR:
+      gnome_error_dialog(_(text));
+      break;
+
+    case PPP_DEBUG:
+      terminal_write(dial, _(text));
+      return;
+      break;
+
+    case PPP_PROMPT_INPUT:
+      if (dial->input_dialog)
+	{
+	  g_error("connect_message_cb(): dial->user_input_dialog != NULL");
+	}
+
+      dial->input_dialog = gnome_request_dialog(
+	                       TRUE,
+			       _(text),
+			       "",
+			       100,
+			       (GnomeStringCallback) input_dialog_cb,
+			       dial,
+			       NULL);
+      
+      set_dial_context(GTK_OBJECT(dial->input_dialog), dial);
+
+      gtk_signal_connect(
+          GTK_OBJECT(dial->input_dialog),
+	  "destroy",
+	  GTK_SIGNAL_FUNC(input_dialog_destroy_cb),
+	  NULL);
+
+      break;
+
+    default:
+      g_error("connect_message_cb(): unknown callback message");
       break;
     }
 
-  /* refresh the dial window if the state changed */
-  if (dial->state != saved_state)
-    {
-      refresh_dial_app(dial);
-    }
-
   /* display a message if there is one */
-  if (text)
+  if (update_statusbar)
     {
-      gnome_appbar_set_status(GNOME_APPBAR(dial->appbar), text);
-    }
-  else
-    {
-      gnome_appbar_set_status(GNOME_APPBAR(dial->appbar), "");
+      if (text)
+	{
+	  gnome_appbar_set_status(GNOME_APPBAR(dial->appbar), _(text));
+	}
+      else
+	{
+	  gnome_appbar_set_status(GNOME_APPBAR(dial->appbar), "");
+	}
     }
 
   /* if the state is disconnected, then unlock the account */
   if (dial->state == _DISCONNECTED)
     {
-      account->locked = FALSE;
+      /* get rid of any remaining input dialog */
+      if (dial->input_dialog)
+	{
+	  gtk_widget_destroy(dial->input_dialog);
+	}
+
+      account_unlock(account);
+      dial->account = NULL;
+    }
+
+
+  /* refresh the dial window if the state changed */
+  if (dial->state != saved_state)
+    {
+      refresh_dial_app(dial);
     }
 }
 
@@ -549,6 +595,20 @@ menu_file_exit_cb(GtkWidget *widget, Dial *dial)
 }
 
 
+static void
+menu_view_debug_terminal_cb(GtkWidget *widget, Dial *dial)
+{
+  if (GTK_CHECK_MENU_ITEM(widget)->active)
+    {
+      terminal_show(dial);
+    }
+  else
+    {
+      terminal_hide(dial);
+    }
+}
+
+
 static void
 menu_account_new_cb(GtkWidget *widget, Dial *dial)
 {
@@ -560,69 +620,49 @@ menu_account_new_cb(GtkWidget *widget, Dial *dial)
 static void
 menu_account_duplicate_cb(GtkWidget *widget, Dial *dial)
 {
+  Account *account;
+
   g_assert(dial != NULL);
 
-  if (dial->account == NULL)
+  if (!(account = get_selected_account(dial)))
     {
       return;
     }
-
-  open_account_window(dial->account, TRUE);
+  open_account_window(account, TRUE);
 }
 
 
 static void
 menu_account_edit_cb(GtkWidget *widget, Dial *dial)
 {
+  Account *account;
+
   g_assert(dial != NULL);
 
-  if (dial->account == NULL)
+  if (!(account = get_selected_account(dial)))
     {
       return;
     }
-
-  open_account_window(dial->account, FALSE);
+  open_account_window(account, FALSE);
 }
 
 
 static void
 menu_account_delete_cb(GtkWidget *widget, Dial *dial)
 {
+  Account *account;
+
   g_assert(dial != NULL);
 
-  if (dial->account == NULL)
+  if (!(account = get_selected_account(dial)))
     {
       return;
     }
 
   /* can't delete locked (in-use) accounts */
-  if (dial->account->locked)
+  if (account_is_locked(account) || !account_delete(account))
     {
-      GString *gstr = g_string_new("");
-
-      g_string_sprintf(
-          gstr,
-	  "Account %s is currently in use and cannot be removed.", 
-	  dial->account->name->str);
-
-      gnome_error_dialog(gstr->str);
-
-      g_string_free(gstr, TRUE);
-      return;
-    }
-
-  /* try to remove the account */
-  if (!account_delete(dial->account))
-    {
-      GString *gstr = g_string_new("");
-      
-      g_string_sprintf(
-	   gstr,
-	  "Account %s cannot be removed.",
-	  dial->account->name->str);
-      gnome_error_dialog(gstr->str);
-
-      g_string_free(gstr, TRUE);
+      gnome_error_dialog(_("The account is in use and cannot be removed."));
       return;
     }
 
@@ -634,17 +674,17 @@ static void
 menu_help_about_cb(GtkWidget *widget, Dial *dial)
 {
   GtkWidget *about;
-  const gchar *authors[] = 
+  const gchar *author[] = 
   {
-    "Jay Painter",
+    GNOME_PPP_AUTHOR,
     NULL
   };
 	
   about = gnome_about_new(
-      "gnome-ppp", 
-      "0.3",
-      "(C) 1998,1999 Jay Painter",
-      authors,
+      _(GNOME_PPP_NAME), 
+      GNOME_PPP_VERSION,
+      _(GNOME_COPYRIGHT),
+      author,
       NULL,
       NULL);
 
@@ -661,6 +701,11 @@ malloc_dial()
   dial->state = _DISCONNECTED;
   dial->timer_id = -1;
 
+  dial->term_app = NULL;
+  dial->term = NULL;
+
+  dial->input_dialog = NULL;
+
   __dial_app_list = g_list_append(__dial_app_list, dial);
   
   return dial;
@@ -688,3 +733,223 @@ set_dial_context(GtkObject *object, Dial *dial)
   gtk_object_set_data(object, "dial_context", dial);
 }
 
+static Account *
+get_selected_account(Dial *dial)
+{
+  Account *account;
+
+  if (GTK_CLIST(dial->account_clist)->selection)
+    {
+      return (Account *)
+	gtk_clist_get_row_data(
+	    GTK_CLIST(dial->account_clist),
+	    GPOINTER_TO_INT(GTK_CLIST(dial->account_clist)->selection->data));
+    }
+  
+  return NULL;
+}
+
+
+/*** terminal ***/
+static void
+new_terminal_app(Dial *dial)
+{
+  GtkWidget *hbox, *scrollbar;
+
+  dial->term_app = gnome_app_new(GNOME_PPP_NAME, "gnome-ppp");
+  set_dial_context(GTK_OBJECT(dial->term_app), dial);
+
+  gtk_signal_connect(
+      GTK_OBJECT(dial->term_app),
+      "delete_event",
+      GTK_SIGNAL_FUNC(terminal_delete_event_cb),
+      NULL);
+
+  /* create hbox */
+  hbox = gtk_hbox_new(FALSE, 0);
+  gnome_app_set_contents(GNOME_APP(dial->term_app), hbox);
+  gtk_box_set_spacing(GTK_BOX(hbox), 2);
+  gtk_container_border_width(GTK_CONTAINER(hbox), 2);
+  gtk_widget_show(hbox);
+
+  /* create terminal */
+  dial->term = zvt_term_new();
+  set_dial_context(GTK_OBJECT(dial->term), dial);
+  gtk_box_pack_start(GTK_BOX(hbox), dial->term, 1, 1, 0);
+  zvt_term_set_shadow_type(ZVT_TERM(dial->term), GTK_SHADOW_IN);
+  zvt_term_set_size(ZVT_TERM(dial->term), 80, 25);
+  zvt_term_set_font_name(ZVT_TERM(dial->term), FONT);
+  zvt_term_set_blink(ZVT_TERM(dial->term), FALSE);
+  zvt_term_set_bell(ZVT_TERM(dial->term), TRUE);
+  zvt_term_set_scrollback(ZVT_TERM(dial->term), SCROLLBACK_LINES);
+  zvt_term_set_scroll_on_keystroke(ZVT_TERM(dial->term), TRUE);
+  zvt_term_set_scroll_on_output(ZVT_TERM(dial->term), FALSE);
+  zvt_term_set_background(ZVT_TERM(dial->term), NULL, 0, 0);
+
+  gtk_signal_connect(
+      GTK_OBJECT(dial->term),
+      "key_press_event",
+      GTK_SIGNAL_FUNC(NULL),
+      NULL);
+
+  gtk_signal_connect_after(
+      GTK_OBJECT(dial->term),
+      "size_allocate",
+      GTK_SIGNAL_FUNC(terminal_size_allocate_cb),
+      NULL);
+
+  gtk_widget_show(dial->term);
+
+  /* scrollbar */
+  scrollbar = 
+    gtk_vscrollbar_new(GTK_ADJUSTMENT(ZVT_TERM(dial->term)->adjustment));
+  GTK_WIDGET_UNSET_FLAGS(scrollbar, GTK_CAN_FOCUS);
+  gtk_box_pack_start(GTK_BOX(hbox), scrollbar, FALSE, TRUE, 0);
+  gtk_widget_show(scrollbar);
+
+  /* realize the GNOME terminal app */
+  gtk_widget_realize(dial->term_app);
+}
+
+
+static void
+terminal_show(Dial *dial)
+{
+  gtk_widget_show(dial->term_app);
+}
+
+
+static void
+terminal_hide(Dial *dial)
+{
+  gtk_widget_hide(dial->term_app);
+}
+
+
+static void
+terminal_write(Dial *dial, gchar *buff)
+{
+  zvt_term_feed(ZVT_TERM(dial->term), buff, strlen(buff));
+#if 0
+  int state;
+
+  state = 0;
+  while (*buff != '\0')
+    {
+      switch (state)
+	{
+	case 0:
+	  if (*buff == '\r')
+	    {
+	      state = 1;
+	      zvt_term_feed(ZVT_TERM(dial->term), "\r\n", 2);
+	    }
+	  else
+	    {
+	      zvt_term_feed(ZVT_TERM(dial->term), buff, 1);
+	    }
+	  break;
+
+	case 1:
+	  if (*buff == '\r')
+	    {
+	      zvt_term_feed(ZVT_TERM(dial->term), "\r\n", 2);
+	    }
+	  else if (*buff == '\n')
+	    {
+	      state = 0;
+	    }
+	  else
+	    {
+	      state = 0;
+	      zvt_term_feed(ZVT_TERM(dial->term), buff, 1);
+	    }
+	  break;
+	}
+
+      buff++;
+    }
+#endif
+}
+
+
+static void
+terminal_size_allocate_cb(GtkWidget *widget)
+{
+  ZvtTerm *term;
+  XSizeHints sizehints;
+  Dial *dial;
+
+  dial = get_dial_context(GTK_OBJECT(widget));
+  term = ZVT_TERM(widget);
+  
+  sizehints.base_width = 
+    (GTK_WIDGET(dial->term_app)->allocation.width) +
+    (GTK_WIDGET(term)->style->klass->xthickness * 2) -
+    (GTK_WIDGET(term)->allocation.width);
+  
+  sizehints.base_height =
+    (GTK_WIDGET(dial->term_app)->allocation.height) +
+    (GTK_WIDGET(term)->style->klass->ythickness * 2) -
+    (GTK_WIDGET(term)->allocation.height);
+  
+  sizehints.width_inc = term->charwidth;
+  sizehints.height_inc = term->charheight;
+  sizehints.min_width = sizehints.base_width + sizehints.width_inc;
+  sizehints.min_height = sizehints.base_height + sizehints.height_inc;
+  
+  sizehints.flags = (PBaseSize|PMinSize|PResizeInc);
+  
+  XSetWMNormalHints(GDK_DISPLAY(),
+		    GDK_WINDOW_XWINDOW (GTK_WIDGET(dial->term_app)->window),
+		    &sizehints);
+  gdk_flush();
+}
+
+
+static gint
+terminal_delete_event_cb(GtkWidget *widget)
+{
+  Dial *dial;
+
+  dial = get_dial_context(GTK_OBJECT(widget));
+  terminal_hide(dial);
+
+  /* update the view/debug window check menu item */
+  gtk_check_menu_item_set_active(
+      GTK_CHECK_MENU_ITEM(dial->debug_terminal_check_menu_item),
+      FALSE);
+
+  return TRUE;
+}
+
+
+static void
+terminal_menu_close_cb(GtkWidget *widget, Dial *dial)
+{
+  terminal_hide(dial);
+}
+
+
+/*** input dialog ***/
+static void
+input_dialog_destroy_cb(GtkWidget *widget)
+{
+  Dial *dial;
+
+  dial = get_dial_context(GTK_OBJECT(widget));
+  dial->input_dialog = NULL;
+}
+
+
+
+static void 
+input_dialog_cb(gchar *string, gpointer data)
+{
+  Dial *dial;
+
+  dial = (Dial *) data;
+  g_assert(dial->account != NULL);
+
+  connect_send(dial->account, string, TRUE);
+}
