@@ -20,22 +20,26 @@
 
 
 /* globals */
-GList *__account_list = NULL;
-GList *__account_list_watchers = NULL;
+GList              *__account_list = NULL;
+static GList       *__account_list_watchers = NULL;
 
 
-/* prototypes */
-static GList   *duplicate_g_string_list(GList *list);
-static void     free_g_string_list(GList **list);
-static void     save_account(Account *account);
-static void     load_account();
-static gchar   *get_string_blank_default(gchar *key);
+/* load/save */
+static void          save_account(Account *account);
+static void          load_account();
 
-ScriptEntry  *script_entry_new(ScriptEntryType type, gchar *text);
-ScriptEntry  *script_entry_clone(ScriptEntry *script_entry);
-void          script_entry_free(ScriptEntry *script_entry);
 
-static void   broadcast_watchers();
+/* script */
+static ScriptEntry  *script_entry_new(ScriptEntryType type, gchar *text);
+static ScriptEntry  *script_entry_clone(ScriptEntry *script_entry);
+static void          script_entry_free(ScriptEntry *script_entry);
+
+
+/* utility */
+static GList        *duplicate_g_string_list(GList *list);
+static void          free_g_string_list(GList **list);
+static gchar        *get_string_blank_default(gchar *key);
+
 
 
 void
@@ -57,11 +61,18 @@ account_new()
 
   account = g_malloc(sizeof(Account));
 
+  /* gint types */
   account->speed                = 115200;
+  account->redial_attempts      = 5;
+  account->mtu                  = 296;
+  account->mru                  = 296;
+
+  /* gboolean types */
   account->default_route        = TRUE;
   account->lock_device          = TRUE;
   account->locked               = FALSE;
 
+  /* GString types */
   account->name                 = g_string_new("");
   account->modem_device         = g_string_new("/dev/modem");
   account->user                 = g_string_new("");
@@ -71,7 +82,10 @@ account_new()
   account->mask                 = g_string_new("");
   account->remote               = g_string_new("");
   account->modem_init           = g_string_new("ATZ");
+  account->connect_program      = g_string_new("");
+  account->disconnect_program   = g_string_new("");
 
+  /* GList types (of GString) */
   account->phone_list           = NULL;
   account->dns_list             = NULL;
   account->domain_list          = NULL;
@@ -90,7 +104,7 @@ account_add(Account *account)
     }
 
   __account_list = g_list_append(__account_list, account);
-  broadcast_watchers();
+  account_broadcast_watchers();
 }
 
 
@@ -110,6 +124,8 @@ account_free(Account *account)
   g_string_free(account->mask, TRUE);
   g_string_free(account->remote, TRUE);
   g_string_free(account->modem_init, TRUE);
+  g_string_free(account->connect_program, TRUE);
+  g_string_free(account->disconnect_program, TRUE);
 
   free_g_string_list(&account->phone_list);
   free_g_string_list(&account->dns_list);
@@ -122,6 +138,59 @@ account_free(Account *account)
 }
 
 
+
+Account *
+account_clone(Account *account)
+{
+  Account *new_account;
+  GList *list;
+  ScriptEntry *script_entry;
+
+  new_account = account_new();
+
+  /* gint types */
+  new_account->speed               = account->speed;
+  new_account->redial_attempts     = 5;
+  new_account->mtu                 = 296;
+  new_account->mru                 = 296;
+
+  /* gboolean types */
+  new_account->default_route       = account->default_route;
+  new_account->lock_device         = account->lock_device;
+
+  /* GString types */
+  g_string_assign(new_account->name,          account->name->str);
+  g_string_assign(new_account->modem_device,  account->modem_device->str);
+  g_string_assign(new_account->user,          account->user->str);
+  g_string_assign(new_account->passwd,        account->passwd->str);
+  g_string_assign(new_account->remotename,    account->remotename->str);
+  g_string_assign(new_account->ip,            account->ip->str);
+  g_string_assign(new_account->mask,          account->mask->str);
+  g_string_assign(new_account->remote,        account->remote->str);
+  g_string_assign(new_account->modem_init,    account->modem_init->str);
+  g_string_assign(new_account->connect_program, account->connect_program->str);
+  g_string_assign(new_account->disconnect_program, account->disconnect_program->str);
+
+  /* GList types (of GString) */
+  new_account->phone_list    = duplicate_g_string_list(account->phone_list);
+  new_account->dns_list      = duplicate_g_string_list(account->dns_list);
+  new_account->domain_list   = duplicate_g_string_list(account->domain_list);
+
+  /* SCRIPT LIST: duplicate all the script entries */
+  list = account->script_list;
+  while (list)
+    {
+     script_entry = (ScriptEntry *) list->data;
+     list = list->next;
+
+     new_account->script_list = 
+       g_list_append(new_account->script_list, script_entry_clone(script_entry));
+    }
+
+  return new_account;
+}
+
+
 gint
 account_delete(Account *account)
 {
@@ -138,49 +207,7 @@ account_delete(Account *account)
   /* remove account from the account list */
   __account_list = g_list_remove(__account_list, account);
   account_free(account);
-  broadcast_watchers();
-}
-
-
-Account *
-account_clone(Account *account)
-{
-  Account *new_account;
-  GList *list;
-  ScriptEntry *script_entry;
-
-  new_account = account_new();
-
-  new_account->speed               = account->speed;
-  new_account->default_route       = account->default_route;
-  new_account->lock_device         = account->lock_device;
-
-  g_string_assign(new_account->name,          account->name->str);
-  g_string_assign(new_account->modem_device,  account->modem_device->str);
-  g_string_assign(new_account->user,          account->user->str);
-  g_string_assign(new_account->passwd,        account->passwd->str);
-  g_string_assign(new_account->remotename,    account->remotename->str);
-  g_string_assign(new_account->ip,            account->ip->str);
-  g_string_assign(new_account->mask,          account->mask->str);
-  g_string_assign(new_account->remote,        account->remote->str);
-  g_string_assign(new_account->modem_init,    account->modem_init->str);
-  
-  new_account->phone_list          = duplicate_g_string_list(account->phone_list);
-  new_account->dns_list            = duplicate_g_string_list(account->dns_list);
-  new_account->domain_list         = duplicate_g_string_list(account->domain_list);
-
-  /* duplicate all the script entries */
-  list = account->script_list;
-  while (list)
-    {
-     script_entry = (ScriptEntry *) list->data;
-     list = list->next;
-
-     new_account->script_list = 
-       g_list_append(new_account->script_list, script_entry_clone(script_entry));
-    }
-
-  return new_account;
+  account_broadcast_watchers();
 }
 
 
@@ -190,6 +217,33 @@ account_list()
   return __account_list;
 }
 
+gint
+account_is_locked(Account *account)
+{
+  return account->locked;
+}
+
+
+gint
+account_lock(Account *account)
+{
+  if (account->locked)
+    {
+      return 0;
+    }
+
+  account->locked = TRUE;
+  return 1;
+}
+
+
+void
+account_unlock(Account *account)
+{
+  account->locked = FALSE;
+}
+
+
 
 void
 account_add_watcher(AccountWatcherCBFunc func)
@@ -205,8 +259,8 @@ account_remove_watcher(AccountWatcherCBFunc func)
 }
 
 
-static void
-broadcast_watchers()
+void
+account_broadcast_watchers()
 {
   GList *list;
   AccountWatcherCBFunc func;
@@ -293,7 +347,6 @@ account_script_list_clear(Account *account)
 }
 
 
-
 void
 account_script_list_append(Account *account, ScriptEntryType type, gchar *text)
 {
@@ -306,38 +359,6 @@ account_script_list_append(Account *account, ScriptEntryType type, gchar *text)
 }
 
 
-ScriptEntry * 
-script_entry_new(ScriptEntryType type, gchar *text)
-{
-  ScriptEntry *script_entry;
-
-  script_entry = g_malloc(sizeof(ScriptEntry));
-  script_entry->type = type;
-  script_entry->text = g_string_new(text);
-  return script_entry;
-}
-
-
-ScriptEntry * 
-script_entry_clone(ScriptEntry *script_entry)
-{
-  ScriptEntry *new_script_entry;
-
-  new_script_entry = g_malloc(sizeof(ScriptEntry));
-  new_script_entry->type = script_entry->type;
-  new_script_entry->text = g_string_new(script_entry->text->str);
-  return new_script_entry;
-}
-
-
-void 
-script_entry_free(ScriptEntry *script_entry)
-{
-  g_string_free(script_entry->text, TRUE);
-  g_free(script_entry);
-}
-
-
 void 
 account_save()
 {
@@ -415,43 +436,6 @@ account_load()
 }
 
 
-/* utility */
-static GList *
-duplicate_g_string_list(GList *list)
-{
-  GList *new_list = NULL;
-  GString *gstr;
-
-  while (list)
-    {
-      gstr = (GString *) list->data;
-      list = list->next;
-      new_list = g_list_append(new_list, g_string_new(gstr->str));
-    }
-
-  return new_list;
-}
-
-
-static void
-free_g_string_list(GList **list)
-{
-  GList *l;
-  GString *gstr;
-
-  l = *list;
-  while (l)
-    {
-      gstr = (GString *) l->data;
-      l = l->next;
-      g_string_free(gstr, TRUE);
-    }
-
-  g_list_free(*list);
-  *list = NULL;
-}
-
-
 static void
 save_string_list(gchar *base_key, GList *list)
 {
@@ -483,9 +467,7 @@ save_account(Account *account)
   key = g_string_new("");
 
   gnome_config_set_string("name", account->name->str);
-  gnome_config_set_string("modem_device", account->modem_device->str);
-  gnome_config_set_int("speed", account->speed);
-
+  gnome_config_set_int("redial_attempts", account->redial_attempts);
   save_string_list("phone_number", account->phone_list);
 
   gnome_config_set_string("user", account->user->str);
@@ -529,12 +511,26 @@ save_account(Account *account)
 	  g_string_sprintf(key, "%d,SE_send_password", i);
 	  gnome_config_set_string(key->str, "");
 	  break;
+
+	case SCRIPT_ENTRY_PROMPT_INPUT:
+	  g_string_sprintf(key, "%d,SE_user_input", i);
+	  gnome_config_set_string(key->str, script_entry->text->str);
+	  break;
 	}
     }
 
-  gnome_config_set_string("modem_init", account->modem_init->str);
   gnome_config_set_bool("default_route", account->default_route);
   gnome_config_set_bool("lock_device", account->lock_device);
+  gnome_config_set_int("mtu", account->mtu);
+  gnome_config_set_int("mru", account->mru);
+  gnome_config_set_string("connect_program", 
+			  account->connect_program->str);
+  gnome_config_set_string("disconnect_program", 
+			  account->disconnect_program->str);
+
+  gnome_config_set_string("modem_init", account->modem_init->str);
+  gnome_config_set_string("modem_device", account->modem_device->str);
+  gnome_config_set_int("speed", account->speed);
 
  cleanup:
   g_string_free(key, TRUE);
@@ -554,8 +550,7 @@ load_account()
   account = account_new();
 
   g_string_assign(account->name, get_string_blank_default("name"));
-  g_string_assign(account->modem_device, get_string_blank_default("modem_device"));
-  account->speed = gnome_config_get_int("speed=15200");
+  account->redial_attempts = gnome_config_get_int("redial_attempts=5");
 
   for (i = 0;; i++)
     {
@@ -638,13 +633,30 @@ load_account()
 	  continue;
 	}
 
+      /* search for user input */
+      g_string_sprintf(key, "%d,SE_user_input", i);
+      if ( (c = gnome_config_get_string(key->str)) != NULL)
+	{
+	  account_script_list_append(account, SCRIPT_ENTRY_PROMPT_INPUT, c);
+	  continue;
+	}
+
       /* if we reach here then there's no more script */
       break;
     }
 
-  g_string_assign(account->modem_init, get_string_blank_default("modem_init"));
   account->default_route = gnome_config_get_bool("default_route=TRUE");
   account->lock_device = gnome_config_get_bool("lock_device=TRUE");
+  account->mtu = gnome_config_get_int("mtu=296");
+  account->mru = gnome_config_get_int("mru=296");
+  g_string_assign(account->connect_program, 
+		  get_string_blank_default("connect_program"));
+  g_string_assign(account->disconnect_program, 
+		  get_string_blank_default("disconnect_program"));
+
+  g_string_assign(account->modem_init, get_string_blank_default("modem_init"));
+  g_string_assign(account->modem_device, get_string_blank_default("modem_device"));
+  account->speed = gnome_config_get_int("speed=15200");
 
  cleanup:
   g_string_free(key, TRUE);
@@ -653,6 +665,40 @@ load_account()
 }
 
 
+/* script sturctures */
+static ScriptEntry * 
+script_entry_new(ScriptEntryType type, gchar *text)
+{
+  ScriptEntry *script_entry;
+
+  script_entry = g_malloc(sizeof(ScriptEntry));
+  script_entry->type = type;
+  script_entry->text = g_string_new(text);
+  return script_entry;
+}
+
+
+static ScriptEntry * 
+script_entry_clone(ScriptEntry *script_entry)
+{
+  ScriptEntry *new_script_entry;
+
+  new_script_entry = g_malloc(sizeof(ScriptEntry));
+  new_script_entry->type = script_entry->type;
+  new_script_entry->text = g_string_new(script_entry->text->str);
+  return new_script_entry;
+}
+
+
+static void 
+script_entry_free(ScriptEntry *script_entry)
+{
+  g_string_free(script_entry->text, TRUE);
+  g_free(script_entry);
+}
+
+
+/* MISC Utility */
 static gchar *
 get_string_blank_default(gchar *key)
 {
@@ -667,121 +713,37 @@ get_string_blank_default(gchar *key)
 }
 
 
-
-
-
-
-
-/*** NEW SAVE STUFF W/O gnome_config ***/
-/* NOT IN USE AT ALL! -JMP */
-static void
-append_string_var(GList **line_list, gchar *variable, gchar *value)
+static GList *
+duplicate_g_string_list(GList *list)
 {
+  GList *new_list = NULL;
   GString *gstr;
 
-  gstr = g_string_new("");
-  g_string_sprintf(gstr, "%s = %s", variable, value);
-  *line_list = g_list_append(*line_list, gstr);
-}
-
-
-static void
-append_int_var(GList **line_list, gchar *variable, gint value)
-{
-  GString *gstr;
-
-  gstr = g_string_new("");
-  g_string_sprintf(gstr, "%s = %d", variable, value);
-  *line_list = g_list_append(*line_list, gstr);
-}
-
-
-static void
-append_string_list_var(GList **line_list, gchar *variable, GList *value_list)
-{
-  GString *gstr;
-
-  gstr = g_string_new("");
-  g_string_sprintf(gstr, "%s list", variable);
-  *line_list = g_list_append(*line_list, gstr);
-
-  while (value_list)
-    {
-      gstr = g_string_new(((GString *) value_list->data)->str);
-      *line_list = g_list_append(*line_list, gstr);
-      value_list = value_list->next;
-    }
-
-  gstr = g_string_new("");
-  g_string_sprintf(gstr, "end %s list", variable);
-  *line_list = g_list_append(*line_list, gstr);
-}
-
-
-static void
-new_save_account(Account *account)
-{
-  gint i;
-  GList *list, *line_list;
-  GString *gstr;
-
-  line_list = NULL;
-
-  /* name, modem device */
-  append_string_var(&line_list, "name", account->name->str);
-  append_string_var(&line_list, "modem_device", account->modem_device->str);
-  append_int_var(&line_list, "speed", account->speed);
-
-  append_string_list_var(&line_list, "phone numbers", account->phone_list);
-
-  append_string_var(&line_list, "user", account->user->str);
-  append_string_var(&line_list, "passwd", account->passwd->str);
-  append_string_var(&line_list, "remotename", account->remotename->str);
-
-  append_string_var(&line_list, "ip", account->ip->str);
-  append_string_var(&line_list, "remote", account->remote->str);
-  append_string_var(&line_list, "mask", account->mask->str);
-
-  append_string_list_var(&line_list, "dns servers", account->dns_list);
-  append_string_list_var(&line_list, "search domains", account->domain_list);
-
-  /* save script -- this is a little hackish */
-  gstr = g_string_new("script list");
-  line_list = g_list_append(line_list, gstr);
-
-  list = account->script_list;
   while (list)
     {
-      ScriptEntry *script_entry;
-
-      script_entry = (ScriptEntry *) list->data;
+      gstr = (GString *) list->data;
       list = list->next;
-
-      gstr = g_string_new("");
-
-      switch (script_entry->type)
-	{
-	case SCRIPT_ENTRY_EXPECT:
-	  g_string_sprintf(gstr, "expect = %s", script_entry->text->str);
-	  break;
-
-	case SCRIPT_ENTRY_SEND:
-	  g_string_sprintf(gstr, "send = %s", script_entry->text->str);
-	  break;
-
-	case SCRIPT_ENTRY_SEND_USER:
-	  g_string_sprintf(gstr, "send username");
-	  break;
-
-	case SCRIPT_ENTRY_SEND_PASSWORD:
-	  g_string_sprintf(gstr, "send password");
-	  break;
-	}
-
-      line_list = g_list_append(line_list, gstr);
+      new_list = g_list_append(new_list, g_string_new(gstr->str));
     }
 
-  gstr = g_string_new("end script list");
-  line_list = g_list_append(line_list, gstr);
+  return new_list;
 }
 
+
+static void
+free_g_string_list(GList **list)
+{
+  GList *l;
+  GString *gstr;
+
+  l = *list;
+  while (l)
+    {
+      gstr = (GString *) l->data;
+      l = l->next;
+      g_string_free(gstr, TRUE);
+    }
+
+  g_list_free(*list);
+  *list = NULL;
+}
