@@ -16,22 +16,28 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+#include <gnome.h>
 #include <stdio.h>
 #include <errno.h>
 #include "gnome-ppp.h"
 #include "misc.h"
 #include "modem.h"
 #include "pppd.h"
+#include "program.h"
 
 
 /* XXX: this is lame. -JMP */
 /* various timeouts in seconds for operations */
-#define GET_TIMEOUT        5
-#define DETECT_TIMEOUT     5
-#define DIAL_TIMEOUT      60
-#define SCRIPT_TIMEOUT    30
-#define PPPD_TIMEOUT       5
-#define FIRST_STATE        0
+#define HANGUP_TIMEOUT        2
+#define GET_TIMEOUT           5
+#define DETECT_TIMEOUT        5
+#define DIAL_TIMEOUT         60
+#define SCRIPT_TIMEOUT       30
+#define PPPD_TIMEOUT          5
+
+#define FIRST_STATE           0
+#define MAX_HANGUP_ATTEMPTS   3
 
 
 /* a couple of macros to make the code more readable */
@@ -49,7 +55,7 @@ enum
   _DIAL,
   _RUN_SCRIPT,
   _EXEC_PPPD,
-  _CONNECTED
+  _CONNECTED,
 };
 
 
@@ -61,18 +67,22 @@ enum
   _MSG_PPPD_FORK,
   _MSG_CLOSING,
   _MSG_MODEM_FD_PASS_TIMEOUT,
+  _MSG_MODEM_HANUP_FAILED,
   _MSG_MODEM_DETECT,
   _MSG_MODEM_DETECT_FAILED,
+  _MSG_MAX_REDIAL,
   _MSG_NO_PHONE_NUMBER,
   _MSG_DIALING,
   _MSG_DIAL_TIMEOUT,
   _MSG_NO_DIALTONE,
   _MSG_BUSY,
   _MSG_CONNECTED,
+  _MSG_NO_CARRIOR,
   _MSG_RUNNING_SCRIPT,
   _MSG_SCRIPT_COMPLETE,
   _MSG_PPP_CONNECTED,
   _MSG_PPP_FAIL,
+  _MSG_PPP_UNKNOWN_DEATH,
   _MSG_LAST_MESSAGE
 };
 
@@ -85,26 +95,34 @@ typedef struct _Connect
   gpointer data;
 
   Modem *modem;
-  GString *session_buffer;
   GString *modem_buffer;
   time_t timeout;
   time_t connect_time;
 
+  gint hanup_attempts;
+
   /* state keepers */
   gint state;
   gint get_state;
   gint detect_state;
   gint dial_state;
+  gint script_state;
   gint pppd_state;
 
+  /* keep track of how many dial attempts there has been */
+  gint dial_attempts;
+
   /* dial phone number list node */
   GList *phone_number_node;
 
   /* current script entry */
   GList *script_entry_node;
+  gboolean send_buffer_set;
+  GString *send_buffer;
 
   gboolean close;
   gboolean dead;
+  gboolean user_closed;
 } Connect;
 
 
@@ -116,21 +134,31 @@ malloc_connect()
   connect->func = NULL;
   connect->data = NULL;
   connect->modem = NULL;
-  connect->session_buffer = g_string_new("");
   connect->modem_buffer = g_string_new("");
   connect->timeout = 0;
   connect->connect_time = 0;
 
+  connect->hanup_attempts = 0;
+
   connect->state = FIRST_STATE;
   connect->get_state = FIRST_STATE;
   connect->detect_state = FIRST_STATE;
   connect->dial_state = FIRST_STATE;
+  connect->script_state = FIRST_STATE;
   connect->pppd_state = FIRST_STATE;
 
+  connect->dial_attempts = 0;
+
   connect->phone_number_node = NULL;
   connect->script_entry_node = NULL;
+
+  connect->send_buffer_set = FALSE;
+  connect->send_buffer = g_string_new("");
+
   connect->close = FALSE;
   connect->dead = FALSE;
+  connect->user_closed = FALSE;
+
   return connect;
 }
 
@@ -140,8 +168,8 @@ free_connect(Connect *connect)
 {
   g_assert(connect != NULL);
 
-  g_string_free(connect->session_buffer, TRUE);
   g_string_free(connect->modem_buffer, TRUE);
+  g_string_free(connect->send_buffer, TRUE);
   g_free(connect);
 }
 
@@ -207,12 +235,18 @@ __init_connect()
       __messages[_MSG_MODEM_FD_PASS_TIMEOUT] =
 	_("The pppd program failed to run gnome-ppp-chat.");
 
+      __messages[_MSG_MODEM_HANUP_FAILED] =
+	_("The modem could not be hung up.");
+
       __messages[_MSG_MODEM_DETECT] =
 	_("Initalizing modem.");
 
       __messages[_MSG_MODEM_DETECT_FAILED] =
 	_("Your modem is not responding to initalization.");
 
+      __messages[_MSG_MAX_REDIAL] = 
+	_("No answer after maximum number of re-dial attempts.");
+
       __messages[_MSG_NO_PHONE_NUMBER] =
 	_("No phone numbers specified for this account.");
 
@@ -227,6 +261,9 @@ __init_connect()
 
       __messages[_MSG_CONNECTED] =
 	_("Connected.");
+      
+      __messages[_MSG_NO_CARRIOR] = 
+	_("The modem lost connection with remote host.");
 
       __messages[_MSG_RUNNING_SCRIPT] =
 	_("Running Script.");
@@ -239,6 +276,9 @@ __init_connect()
 
       __messages[_MSG_PPP_FAIL] =
 	_("Failed to establish a PPP connection.");
+
+      __messages[_MSG_PPP_UNKNOWN_DEATH] =
+	_("The pppd daemon died unexpectedly.");
     }
 }
 
@@ -269,6 +309,7 @@ connect_start(Account *account, ConnectCBFunc func, gpointer data)
   if (pppd_exec(connect->account, pppd_die_cb, connect))
     {
       CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_PPPD_FORK]);
+      CALLBACK(connect, PPP_DEBUG, _("\r\n---NEW CONNECTION---\r\n"));
     }
   else
     {
@@ -288,7 +329,7 @@ connect_start(Account *account, ConnectCBFunc func, gpointer data)
 	}
     }
 
-  return 1;
+  return pppd_pid(connect->account);
 
  error:
   /* remove connection from hash */
@@ -340,10 +381,39 @@ connect_stop(Account *account)
     {
       CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_CLOSING]);
     }
+
   connect->close = TRUE;
+  connect->user_closed = TRUE;
 }
 
 
+void
+connect_send(Account *account, gchar *text, gboolean append_cr)
+{
+  Connect *connect;
+
+  g_assert(account != NULL);
+  
+  connect = g_hash_table_lookup(__connect_hash, account);
+  if (!connect)
+    {
+      return;
+    }
+
+  connect->send_buffer_set = TRUE;
+  
+  if (append_cr)
+    {
+      g_string_sprintf(connect->send_buffer, "%s\r\n", text);
+    }
+  else
+    {
+      g_string_assign(connect->send_buffer, text);
+    }
+}
+
+
+
 /* this is the main state machine for the ENTIRE connection cycle */
 static gint
 state_engine_iteration(Connect *connect)
@@ -394,12 +464,25 @@ state_engine_iteration(Connect *connect)
 	  pppd_kill(connect->account);
 	}
 
+      /* we're really done */
       if (connect->dead)
 	{
+	  if (!connect->user_closed)
+	    {
+	      CALLBACK(connect, PPP_ERROR, __messages[_MSG_PPP_UNKNOWN_DEATH]);
+	    }
+
+	  /* run the disconnection program */
+	  if (connect->state == _CONNECTED &&
+	      connect->account->disconnect_program->len > 0)
+	    {
+	      program_exec(connect->account,
+			   connect->account->disconnect_program->str);
+	    }
+
 	  goto destroy;
 	}
     }
-
   return 1;
 
  destroy:
@@ -435,6 +518,8 @@ state_get_modem(Connect *connect)
 	    {
 	      g_error("state_get_modem(): ioctl failure on modem");
 	    }
+
+	  modem_drain(connect->modem);
 	  NEXT_STATE(connect->state);
 	}
       else
@@ -446,6 +531,7 @@ state_get_modem(Connect *connect)
 		{
 		  CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_FD_PASS_TIMEOUT]);
 		  connect->close = TRUE;
+		  connect->user_closed = TRUE;
 		  goto error;
 		}
 	      break;
@@ -473,6 +559,9 @@ state_detect_modem(Connect *connect)
 
   enum
   {
+    __CHECK_CARRIER,
+    __START_PAUSE,
+    __END_PAUSE,
     __AT_INIT,
     __OK
   };
@@ -481,6 +570,51 @@ state_detect_modem(Connect *connect)
 
   switch (connect->detect_state)
     {
+    case __CHECK_CARRIER:
+      if (modem_carrier(connect->modem))
+	{
+	  /* if we're under MAX_HANGUP_ATTEMPTS, then try to hanup the
+	   * modem again; otherwise error out and quit
+	   */
+	  if (connect->hanup_attempts++ < MAX_HANGUP_ATTEMPTS)
+	    {
+	      modem_hangup(connect->modem);
+	      NEXT_STATE(connect->detect_state);
+	    }
+	  else
+	    {
+	      CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_HANUP_FAILED]);
+	      connect->close = TRUE;
+	      connect->user_closed = TRUE;
+	    }
+	}
+      else
+	{
+	  NEXT_STATE(connect->detect_state);
+	}
+      break;
+
+
+    case __START_PAUSE:
+      SET_TIMEOUT(connect->timeout, HANGUP_TIMEOUT);
+      NEXT_STATE(connect->detect_state);
+      break;
+
+
+    case __END_PAUSE:
+      if (TIMEOUT(connect->timeout))
+	{
+	  if (modem_carrier(connect->modem))
+	    {
+	      connect->detect_state = __CHECK_CARRIER;
+	    }
+	  else
+	    {
+	      NEXT_STATE(connect->detect_state);
+	    }
+	}
+
+
     case __AT_INIT:
       CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_MODEM_DETECT]);
 
@@ -500,7 +634,6 @@ state_detect_modem(Connect *connect)
        */
       if (search_modem_buffer(connect, "OK\r"))
 	{
-	  modem_drain(connect->modem);
 	  NEXT_STATE(connect->state);
 	}
       else
@@ -512,6 +645,7 @@ state_detect_modem(Connect *connect)
 	    {
 	      CALLBACK(connect, PPP_ERROR, __messages[_MSG_MODEM_DETECT_FAILED]);
 	      connect->close = TRUE;
+	      connect->user_closed = TRUE;
 	    }
 	}
       break;
@@ -540,12 +674,21 @@ state_dial(Connect *connect)
   switch (connect->dial_state)
     {
     case __CALL:
+      /* check for maximum # of redials */
+      if (connect->dial_attempts >= connect->account->redial_attempts)
+	{
+	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_MAX_REDIAL]);
+	  connect->close = TRUE;
+	  connect->user_closed = TRUE;
+	  goto cleanup;
+	}
 
       /* the account HAS to have at least one phone number to dial */
       if (g_list_length(connect->account->phone_list) == 0)
 	{
 	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_PHONE_NUMBER]);
 	  connect->close = TRUE;
+	  connect->user_closed = TRUE;
 	  goto cleanup;
 	}
       
@@ -565,10 +708,10 @@ state_dial(Connect *connect)
 	{
 	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_DIALING]);
 
-	  modem_drain(connect->modem);
-
 	  g_string_sprintf(gstr, "ATDT%s\r", phone_number->str);
 	  modem_write(connect->modem, gstr->str, gstr->len);
+	  connect->dial_attempts++;
+
 	  SET_TIMEOUT(connect->timeout, DIAL_TIMEOUT);
 	  connect->dial_state = __ANSWER;
 	}
@@ -597,6 +740,7 @@ state_dial(Connect *connect)
 	{
 	  CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_DIALTONE]);
 	  connect->close = TRUE;
+	  connect->user_closed = TRUE;
 	  goto cleanup;
 	}
 
@@ -634,22 +778,47 @@ state_dial(Connect *connect)
 static void
 state_run_script(Connect *connect)
 {
-  gboolean next;
   ScriptEntry *script_entry;
   GString *gstr;
 
+  enum
+  {
+    __INITALIZE,
+    __RUN_SCRIPT,
+    __WAIT_EXPECT,
+    __WAIT_USER_INPUT
+  };
+
+
   gstr = g_string_new("");
 
-  /* initalize the script entry glist to the first
-   * script entry in the account
-   */
-  if (!connect->script_entry_node)
+  /* check for carrier detect */
+  if (!modem_carrier(connect->modem))
     {
-      CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_RUNNING_SCRIPT]);
+      CALLBACK(connect, PPP_ERROR, __messages[_MSG_NO_CARRIOR]);
+      connect->close = TRUE;
+      connect->user_closed = TRUE;
+      goto cleanup;
+    }
 
+  /* script entry data off of our current node */
+  if (connect->script_entry_node)
+    {
+      script_entry = (ScriptEntry *) connect->script_entry_node->data;
+      if (!script_entry)
+	{
+	  g_error("state_run_script(): script entry data==NULL");
+	}
+    }
+
+  switch (connect->script_state)
+    {
+    case __INITALIZE:
       if (connect->account->script_list)
 	{
+	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_RUNNING_SCRIPT]);
 	  connect->script_entry_node = connect->account->script_list;
+	  NEXT_STATE(connect->script_state);
 	}
       else
 	{
@@ -657,61 +826,82 @@ state_run_script(Connect *connect)
 	   * to the next connection state
 	   */
 	  NEXT_STATE(connect->state);
-	  goto cleanup;
 	}
-    }
+      break;
 
-  next = FALSE;
-  script_entry = (ScriptEntry *) connect->script_entry_node->data;
 
-  switch (script_entry->type)
-    {
-    case SCRIPT_ENTRY_EXPECT:
+    case __RUN_SCRIPT:
+      switch (script_entry->type)
+	{
+	case SCRIPT_ENTRY_EXPECT:
+	  connect->script_state = __WAIT_EXPECT;
+	  break;
+
+	case SCRIPT_ENTRY_SEND:
+	  g_string_sprintf(gstr, "%s\r\n", script_entry->text->str);
+	  modem_write(connect->modem, gstr->str, gstr->len);
+	  goto next;
+	  break;
+	  
+	case SCRIPT_ENTRY_SEND_USER:
+	  g_string_sprintf(gstr, "%s\r\n", connect->account->user->str);
+	  modem_write(connect->modem, gstr->str, gstr->len);
+	  goto next;
+	  break;
+	  
+	case SCRIPT_ENTRY_SEND_PASSWORD:
+	  g_string_sprintf(gstr, "%s\r\n", connect->account->passwd->str);
+	  modem_write(connect->modem, gstr->str, gstr->len);
+	  goto next;
+	  break;
+	  
+	case SCRIPT_ENTRY_PROMPT_INPUT:
+	  CALLBACK(connect, PPP_PROMPT_INPUT, script_entry->text->str);
+	  connect->script_state = __WAIT_USER_INPUT;
+	  break;
+	}
+      break;
+
+
+    case __WAIT_EXPECT:
       if (search_modem_buffer(connect, script_entry->text->str))
 	{
-	  next = TRUE;
+	  connect->script_state = __RUN_SCRIPT;
+	  goto next;
 	}
       break;
 
 
-    case SCRIPT_ENTRY_SEND:
-      g_string_sprintf(gstr, "%s\r\n", script_entry->text->str);
-      modem_write(connect->modem, gstr->str, gstr->len);
-      next = TRUE;
-      break;
-
-
-    case SCRIPT_ENTRY_SEND_USER:
-      g_string_sprintf(gstr, "%s\r\n", connect->account->user->str);
-      modem_write(connect->modem, gstr->str, gstr->len);
-      next = TRUE;
-      break;
-
-      
-    case SCRIPT_ENTRY_SEND_PASSWORD:
-      g_string_sprintf(gstr, "%s\r\n", connect->account->passwd->str);
-      modem_write(connect->modem, gstr->str, gstr->len);
-      next = TRUE;
-      break;
-    }
-
-  /* get the next element of the script to execute,
-   * and if there's no more go to the next state of the connect
-   * state engine
-   */
-  if (next)
-    {
-      connect->script_entry_node = connect->script_entry_node->next;
- 
-      if (connect->script_entry_node == NULL)
+    case __WAIT_USER_INPUT:
+      if (connect->send_buffer_set)
 	{
-	  CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_SCRIPT_COMPLETE]);
-	  NEXT_STATE(connect->state);
+	  modem_write(
+	      connect->modem, 
+	      connect->send_buffer->str,
+	      connect->send_buffer->len);
+
+	  connect->send_buffer_set = FALSE;
+	  connect->script_state = __RUN_SCRIPT;
+	  goto next;
 	}
+      break;
     }
 
  cleanup:
   g_string_free(gstr, TRUE);
+  return;
+
+ next:
+  /* get next script node */
+  connect->script_entry_node = connect->script_entry_node->next;
+  if (!connect->script_entry_node)
+    {
+      CALLBACK(connect, PPP_IN_PROGRESS, __messages[_MSG_SCRIPT_COMPLETE]);
+      NEXT_STATE(connect->state);
+    }
+
+  g_string_free(gstr, TRUE);
+  return;
 }
 
 
@@ -723,6 +913,7 @@ state_exec_pppd(Connect *connect)
   enum
   {
     __START_PPP,
+    __PAUSE,
     __DETECT_PPP_CONNECTION
   };
   
@@ -739,7 +930,7 @@ state_exec_pppd(Connect *connect)
        */
       if (pppd_end_chat(connect->account))
 	{
-	  SET_TIMEOUT(connect->timeout, PPPD_TIMEOUT);
+	  SET_TIMEOUT(connect->timeout, 2);
 	  NEXT_STATE(connect->pppd_state);
 	}
       else
@@ -757,10 +948,26 @@ state_exec_pppd(Connect *connect)
       break;
 
 
+    case __PAUSE:
+      if (TIMEOUT(connect->timeout))
+	{
+	  SET_TIMEOUT(connect->timeout, PPPD_TIMEOUT);
+	  NEXT_STATE(connect->pppd_state);
+	}
+
+
     case __DETECT_PPP_CONNECTION:
       if (pppd_device_number(connect->account, &device_number))
 	{
 	  CALLBACK(connect, PPP_CONNECTED, __messages[_MSG_PPP_CONNECTED]);
+
+	  /* run connect program */
+	  if (connect->account->connect_program->len > 0)
+	    {
+	      program_exec(connect->account,
+			   connect->account->connect_program->str);
+	    }
+
 	  NEXT_STATE(connect->state);
 	}
       else
@@ -772,6 +979,7 @@ state_exec_pppd(Connect *connect)
 		{
 		  CALLBACK(connect, PPP_ERROR, __messages[_MSG_PPP_FAIL]);
 		  connect->close = TRUE;
+		  connect->user_closed = TRUE;
 		  goto error;
 		}
 	      break;
@@ -794,29 +1002,6 @@ state_exec_pppd(Connect *connect)
 static void
 state_connected(Connect *connect)
 {
-#if 0
-  GString *gstr;
-  gint total_seconds, hours, minutes;
-
-  gstr = g_string_new("");
-
-  /* set the inital time of connect */
-  if (connect->connect_time == 0)
-    {
-      connect->connect_time = time(NULL);
-    }
-
-  total_seconds = time(NULL) - connect->connect_time;
-
-  hours = total_seconds / 3600;
-  minutes = (total_seconds % 3600) / 60;
-
-  g_string_sprintf(gstr, "Connected %d:%d", hours, minutes);
-  CALLBACK(connect, PPP_CONNECTED, gstr->str);
-
- cleanup:
-  g_string_free(gstr, TRUE);
-#endif
 }
 
 
@@ -858,19 +1043,21 @@ static gboolean
 read_charactor(Connect *connect)
 {
   gint len;
-  gchar c;
+  gchar buff[2];
 
   if (modem_read_ready(connect->modem))
     {
-      len = modem_read(connect->modem, &c, 1);
+      len = modem_read(connect->modem, buff, 1);
       if (len == 1)
 	{
-	  g_string_append_c(connect->session_buffer, c);
-	  g_string_append_c(connect->modem_buffer, c);
+	  buff[1] = '\0';
+	  CALLBACK(connect, PPP_DEBUG, buff);
+	  g_string_append_c(connect->modem_buffer, buff[0]);
 	}
       else
 	{
 	  connect->close = TRUE;
+	  connect->user_closed = TRUE;
 	  return FALSE;
 	}
 
