@@ -28,7 +28,8 @@ typedef enum
   SCRIPT_ENTRY_EXPECT,
   SCRIPT_ENTRY_SEND,
   SCRIPT_ENTRY_SEND_USER,
-  SCRIPT_ENTRY_SEND_PASSWORD
+  SCRIPT_ENTRY_SEND_PASSWORD,
+  SCRIPT_ENTRY_PROMPT_INPUT
 }
 ScriptEntryType;
 
@@ -37,7 +38,9 @@ typedef enum
   PPP_DISCONNECTED,
   PPP_IN_PROGRESS,
   PPP_CONNECTED,
-  PPP_ERROR
+  PPP_ERROR,
+  PPP_DEBUG,
+  PPP_PROMPT_INPUT
 } PPPMessageType;
 
 
@@ -55,8 +58,7 @@ struct _ScriptEntry
 struct _Account 
 {
   GString *name;             /* account name */
-  GString *modem_device;     /* device path */
-  gint speed;                /* connection speed */
+  gint redial_attempts;      /* number of times to redial before failing */
   GList *phone_list;         /* phone number list */
 
   /* for scripting, PAP, CHAP and other uses */
@@ -80,22 +82,29 @@ struct _Account
    */
   GList *script_list;
 
-  /* advanced settings */
-  GString *modem_init;
+  /* system settings */
   gboolean default_route;
   gboolean lock_device;
+  gint mtu;
+  gint mru;
+  GString *connect_program;
+  GString *disconnect_program;
 
-  /* application state information */
+  GString *modem_init;
+  GString *modem_device;
+  gint speed;
+
+  /* PRIVATE */
   gboolean locked;
 };
 
 
 /* typedefs */
 typedef void (*AccountWatcherCBFunc)();
-typedef void (*ConnectCBFunc)(Account *account, 
-			      PPPMessageType message,
-			      gchar *text,
-			      gpointer data);
+typedef void (*ConnectCBFunc)(Account         *account, 
+			      PPPMessageType   message,
+			      gchar           *text,
+			      gpointer         data);
 
 
 /* initalize library -- a must before using! */
@@ -108,8 +117,16 @@ void          account_save();
 
 /* list of all accounts */
 GList        *account_list();
+
+/* account locking */
+gint          account_is_locked(Account *account);
+gint          account_lock(Account *account);
+void          account_unlock(Account *account);
+
+/* add watcher functions for changes/updates to accounts */
 void          account_add_watcher(AccountWatcherCBFunc func);
 void          account_remove_watcher(AccountWatcherCBFunc func);
+void          account_broadcast_watchers();
 
 /* allocate account structures */
 Account      *account_new();
@@ -148,4 +165,8 @@ gint          connect_engine_iteration(Account *account);
  */
 void          connect_stop(Account *account);
 
+/* send output after the connect engine prompts for user input */
+void          connect_send(Account *account, gchar *text, gboolean append_cr);
+
+
 #endif /* __gnome_ppp_h__ */
