@@ -19,7 +19,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <signal.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -29,10 +28,10 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <sys/uio.h>
 
 #include "config.h"
 
-#include "gnome-ppp.h"
 #include "misc.h"
 #include "pppd.h"
 #include "dns.h"
@@ -120,20 +119,15 @@ free_ppp(PPP *ppp)
 gint                            pppd_errno;
 static GHashTable              *__ppp_hash = NULL;
 static GHashTable              *__ppp_pid_hash = NULL;
-static struct sigaction         sact;
-static gint                     block_sigchld_refcount = 0;
 
 
 /* prototypes */
 static pid_t fork_pppd(PPP *ppp);
-static void handle_sigchld(int signum);
-static void block_sigchld();
-static void unblock_sigchld();
-static gint check_device_lock(gchar *device_name);
-static gint check_pid_file(PPP *ppp);
+static gint  check_device_lock(gchar *device_name);
+static gint  check_pid_file(PPP *ppp);
 static pid_t get_pid(gint device_number);
-static gint is_pid_filename(gchar *filename);
-static int recieve_fd(int fd, int *recvfd);
+static gint  is_pid_filename(gchar *filename);
+static int   recieve_fd(int fd, int *recvfd);
 
 
 
@@ -151,12 +145,6 @@ __init_ppp()
 
       /* map pppd pids to PPP structures */
       __ppp_pid_hash = g_hash_table_new(g_int_hash, g_int_equal);
-
-      /* catch SIGCHLD to detect dead PPP connections */
-      sact.sa_handler = handle_sigchld;
-      sigemptyset(&sact.sa_mask);
-      sigaddset(&sact.sa_mask, SIGCHLD);
-      sigaction(SIGCHLD, &sact, (struct sigaction *) NULL);
     }
 }
 
@@ -443,159 +431,11 @@ pppd_pid(Account *account)
 }
 
 
-static pid_t
-fork_pppd(PPP *ppp)
+void 
+pppd_sigchld(pid_t pid, int status)
 {
-  pid_t pid;
-  int arg_index;
-  char *args[MAX_ARGS];
-  GString *device, *speed, *user, *remotename, *chat, *netmask, *ip;
-
-  arg_index = 0;
-
-  device = g_string_new("");
-  speed = g_string_new("");
-  user = g_string_new("");
-  remotename = g_string_new("");
-  chat = g_string_new("");
-  netmask = g_string_new("");
-  ip = g_string_new("");
-
-  /* name of the program as argv[0] */
-  args[arg_index++] = "pppd";
-  
-  /* modem device */
-  g_string_sprintf(device, "%s", ppp->account->modem_device->str);
-  args[arg_index++] = device->str;
-  
-  /* speed */
-  g_string_sprintf(speed, "%d", ppp->account->speed);
-  args[arg_index++] = speed->str;
-  
-  /* local:remote ip address */
-  if (ppp->account->ip->len > 0 || ppp->account->remote->len > 0)
-    {
-      if (ppp->account->ip->len > 0)
-	{
-	  g_string_assign(ip, ppp->account->ip->str);
-	}
-      else
-	{
-	  g_string_assign(ip, "0.0.0.0");
-	}
-
-      g_string_append_c(ip, ':');
-
-      if (ppp->account->remote->len > 0)
-      	{
-	  g_string_append(ip, ppp->account->remote->str);
-	}
-      else
-	{
-	  g_string_append(ip, "0.0.0.0");
-	}
-
-      args[arg_index++] = ip->str;
-    }
-
-  /* netmask */
-  if (ppp->account->mask->len > 0)
-    {
-      args[arg_index++] = "netmask";
-      g_string_sprintf(netmask, "%s", ppp->account->mask->str);
-      args[arg_index++] = netmask->str;
-    }
-  
-  /* lock the device */
-  if (ppp->account->lock_device)
-    {
-      args[arg_index++] = "lock";
-    }
-
-  /* don't fork off */
-  args[arg_index++] = "-detach";
-  
-  /* add default route */
-  if (ppp->account->default_route)
-    {
-      args[arg_index++] = "defaultroute";
-    }
-
-  /* use CRTSCTS flow control */
-  args[arg_index++] = "crtscts";
-  
-  /* PAP/CHAP */
-  if (ppp->account->remotename->len > 0 &&
-      ppp->account->user->len > 0)
-    {
-      args[arg_index++] = "user";
-      
-      g_string_sprintf(user, "%s", ppp->account->user->str);
-      args[arg_index++] = user->str;
-      
-      args[arg_index++] = "remotename";
-
-      g_string_sprintf(remotename, "%s", ppp->account->remotename->str);
-      args[arg_index++] = remotename->str;
-    }
-      
-  /* gnome-ppp-chat */
-  args[arg_index++] = "connect";
-  g_string_sprintf(chat, "gnome-ppp-chat %s", ppp->socket_path->str);
-  args[arg_index++] = chat->str;
-  
-  /* sentinal */  
-  args[arg_index] = NULL;
-
-  pid = fork();
-  if (pid == 0)
-    {
-      int fd;
-
-      /* become process group leader so pppd's bad habbit of
-       * echoing signals to its entire fucking process group
-       * doesn't clobber this program
-       */
-      if (setpgrp() == -1)
-	{
-	  _exit(0);
-	}
-
-      /* close all file descriptors */
-      for (fd = 0; fd <= 64; fd++)
-	{
-	  close(fd);
-	}
-
-      /* if the execv fails, exit with _exit so no
-       * atexit handlers get called
-       */
-      execv(PPPD_PATH, args);
-      _exit(0);
-    }
-
- cleanup:
-  g_string_free(device, TRUE);
-  g_string_free(speed, TRUE);
-  g_string_free(user, TRUE);
-  g_string_free(remotename, TRUE);
-  g_string_free(chat, TRUE);
-  g_string_free(netmask, TRUE);
-  g_string_free(ip, TRUE);
-
-  return pid;
-}
-
-
-static void 
-handle_sigchld(int signum)
-{
-  pid_t pid;
-  int status;
   PPP *ppp;
 
-  pid = wait(&status);
-
   ppp = g_hash_table_lookup(__ppp_pid_hash, &pid);
   if (ppp)
     {
@@ -627,37 +467,152 @@ handle_sigchld(int signum)
       CALLBACK(ppp);
       g_free(ppp);
     }
-
-  /* reset the signal */
-  sigaction(SIGCHLD, &sact, (struct sigaction *) NULL);
 }
 
 
-static void
-block_sigchld()
+static pid_t
+fork_pppd(PPP *ppp)
 {
-  block_sigchld_refcount++;
+  pid_t pid;
 
-  g_assert(block_sigchld_refcount > 0);
-
-  if (block_sigchld_refcount == 1)
+  pid = fork();
+  if (pid == 0)
     {
-      sigprocmask(SIG_BLOCK, &sact.sa_mask, (sigset_t *) NULL);
+      int fd, arg_index;
+      char *args[MAX_ARGS];
+      GString *device, *speed, *user, *remotename, *chat, *netmask, *ip,
+	*mtu, *mru;
+
+      arg_index = 0;
+      
+      device = g_string_new("");
+      speed = g_string_new("");
+      user = g_string_new("");
+      remotename = g_string_new("");
+      chat = g_string_new("");
+      netmask = g_string_new("");
+      ip = g_string_new("");
+      mtu = g_string_new("");
+      mru = g_string_new("");
+      
+      /* name of the program as argv[0] */
+      args[arg_index++] = "pppd";
+      
+      /* modem device */
+      g_string_sprintf(device, "%s", ppp->account->modem_device->str);
+      args[arg_index++] = device->str;
+      
+      /* speed */
+      g_string_sprintf(speed, "%d", ppp->account->speed);
+      args[arg_index++] = speed->str;
+      
+      /* local:remote ip address */
+      if (ppp->account->ip->len > 0 || ppp->account->remote->len > 0)
+	{
+	  if (ppp->account->ip->len > 0)
+	    {
+	      g_string_assign(ip, ppp->account->ip->str);
+	    }
+	  else
+	    {
+	      g_string_assign(ip, "0.0.0.0");
+	    }
+	  
+	  g_string_append_c(ip, ':');
+	  
+	  if (ppp->account->remote->len > 0)
+	    {
+	      g_string_append(ip, ppp->account->remote->str);
+	    }
+	  else
+	    {
+	      g_string_append(ip, "0.0.0.0");
+	    }
+	  
+	  args[arg_index++] = ip->str;
+	}
+      
+      /* netmask */
+      if (ppp->account->mask->len > 0)
+	{
+	  args[arg_index++] = "netmask";
+	  g_string_sprintf(netmask, "%s", ppp->account->mask->str);
+	  args[arg_index++] = netmask->str;
+	}
+      
+      /* lock the device */
+      if (ppp->account->lock_device)
+	{
+	  args[arg_index++] = "lock";
+	}
+      
+      /* don't fork off */
+      args[arg_index++] = "-detach";
+      
+      /* add default route */
+      if (ppp->account->default_route)
+	{
+	  args[arg_index++] = "defaultroute";
+	}
+      
+      /* mtu/mru (Quake optimization support) */
+      args[arg_index++] = "mtu";
+      g_string_sprintf(mtu, "%d", ppp->account->mtu);
+      args[arg_index++] = mtu->str;
+      
+      args[arg_index++] = "mru";
+      g_string_sprintf(mru, "%d", ppp->account->mru);
+      args[arg_index++] = mru->str;
+      
+      /* use CRTSCTS flow control */
+      args[arg_index++] = "crtscts";
+      
+      /* PAP/CHAP */
+      if (ppp->account->remotename->len > 0 &&
+	  ppp->account->user->len > 0)
+	{
+	  args[arg_index++] = "user";
+	  
+	  g_string_sprintf(user, "%s", ppp->account->user->str);
+	  args[arg_index++] = user->str;
+	  
+	  args[arg_index++] = "remotename";
+	  
+	  g_string_sprintf(remotename, "%s", ppp->account->remotename->str);
+	  args[arg_index++] = remotename->str;
+	}
+      
+      /* gnome-ppp-chat */
+      args[arg_index++] = "connect";
+      g_string_sprintf(chat, "gnome-ppp-chat %s", ppp->socket_path->str);
+      args[arg_index++] = chat->str;
+      
+      /* sentinal */  
+      args[arg_index] = NULL;
+      
+      /* become process group leader so pppd's bad habbit of
+       * echoing signals to its entire fucking process group
+       * doesn't clobber this program
+       */
+      if (setpgrp() == -1)
+	{
+	  _exit(0);
+	}
+
+      /* close all file descriptors */
+      for (fd = 0; fd <= 64; fd++)
+	{
+	  close(fd);
+	}
+
+      /* if the execv fails, exit with _exit so no
+       * atexit handlers get called
+       */
+      execv(PPPD_PATH, args);
+      _exit(0);
     }
-}
 
-
-static void
-unblock_sigchld()
-{
-  block_sigchld_refcount--;
-
-  g_assert(block_sigchld_refcount >= 0);
-
-  if (block_sigchld_refcount == 0)
-    {
-      sigprocmask(SIG_UNBLOCK, &sact.sa_mask, (sigset_t *) NULL);
-    }
+  return pid;
 }
 
 
